<?php

class Util {

    public $badwords;

    function __construct() {

        $this->badwords = array();
    }

    function prepareBadword(String $pathToBadwordList) {

        $handle = fopen($pathToBadwordList, "r");
        if ($handle) {
            while (($lineItem = fgets($handle)) !== false) {

                // GET RID OF ALL FRILLS AND MAKE IT UPPERCASE
                $badword = trim(strtoupper($lineItem));

                // EMPTY?
                if (empty($badword)) {

                    continue;
                }

                // CHECK IF BADWORD IS WITHIN CODE_LENGTH CONSTRAINT
                if (strlen($badword) > CODE_LENGTH) {

                    continue;
                }

                // CHECK IF BADWORD CAN BE BUILT USING CODE_CHARSET
                for ($i = 0; $i < strlen($badword); $i++) {

                    if (strpos($this->deleetString(CODE_CHARSET), $badword[$i]) === false) {

                        continue 2;
                    }
                }

                $this->badwords[] = $badword;
            }

            fclose($handle);

            // REMOVE DUPLICATES
            $this->badwords = array_unique($this->badwords);

            // SORT BY LENGTH
            usort($this->badwords, function($a, $b) {

                return strlen($a) - strlen($b) ?: strcmp($a, $b);
            });

            // REMOVE WORDS THAT CAN BE BUILT USING SHORTER BADWORDS
            for ($i = 0; $i < count($this->badwords); $i++) {

                if (!isset($this->badwords[$i])) {

                    continue;
                }

                for ($j = ($i + 1); $j < count($this->badwords); $j++) {

                    if (!isset($this->badwords[$j])) {
                        continue;
                    }

                    if (strpos(($this->badwords[$j]), $this->badwords[$i])) {

                        unset($this->badwords[$j]);
                    }
                }
            }

            // CLEAN UP ARRAY. REMOVE ALL UNSETS
            $this->badwords = array_values($this->badwords);
            Logger::log(count($this->badwords) . " badwords imported.");
        } else {

            $this->log("Badword dictionary not found");
            exit();
        }
    }

    function isBadword($code) {

        foreach ($this->badwords as $badword) {

            $check = strtoupper($this->deleetString($code));

            if (strpos(($check), $badword) !== false) {

                return true;
            }
        }

        return false;
    }

    function isWithinEntropyThreshold($code, $lowerThreshold, $upperThreshold) {

        $h = 0;
        $size = strlen($code);
        foreach (count_chars($code, 1) as $v) {
            $p = $v / $size;
            $h -= $p * log($p) / log(2);
        }

        return (($lowerThreshold <= $h) && ($h <= $upperThreshold)) ? true : false;
    }

    function deleetString(String $stringToBeDeleeted) {

        return str_replace(array('1', '2', '3', '4', '5', '6', '7', '8', '9', '0'), array('I', 'Z', 'E', 'A', 'S', 'G', 'T', 'B', 'G', 'O'), $stringToBeDeleeted);
    }

    function generateCode(int $length, String $charset) {

        $charsetLength = strlen($charset);
        $code = '';

        for ($i = 0; $i < $length; $i++) {

            $code .= $charset[random_int(0, $charsetLength - 1)];
        }

        return $code;
    }

}
