<?php

set_time_limit(0);

include 'include/CodeWorker.php';
include 'include/CodeTask.php';
include 'include/Config.php';
include 'include/Database.php';
include 'include/Logger.php';
include 'include/Util.php';

Logger::log("NUMBER_OF_CODES = " . NUMBER_OF_CODES . "; CODE_LENGTH = " . CODE_LENGTH . "; CODE_CHARSET = " . CODE_CHARSET);
Logger::log("DB_DELETE_ON_START = " . DB_DELETE_ON_START . "; DB_UNIQUE_CODE_CHECK = " . DB_UNIQUE_CODE_CHECK . "; DB_NAME = " . DB_PATH . "; DB_CODE_COLUMN_NAME = " . DB_CODE_COLUMN_NAME);
Logger::log("BADWORD_CHECK = " . BADWORD_CHECK . "; BADWORD_DICT = " . BADWORD_DICT);
Logger::log("ENTROPY_CHECK = " . ENTROPY_CHECK . "; ENTROPY_LOWER_THRESHOLD = " . ENTROPY_LOWER_THRESHOLD . "; ENTROPY_UPPER_THRESHOLD = " . ENTROPY_UPPER_THRESHOLD);
Logger::log("NUMBER_OF_WORKERS_IN_POOL = " . NUMBER_OF_WORKERS_IN_POOL . "; NUMBER_OF_CONCURRENT_TASKS = " . NUMBER_OF_CONCURRENT_TASKS);
Logger::log("MAX_BUFFER_SIZE_BEFORE_RELEASING_TASK = " . BUFFER_SIZE_BEFORE_RELEASING_TASK . "; MAX_BUFFER_SIZE_BEFORE_SENDING_TO_DB = " . BUFFER_SIZE_BEFORE_SENDING_TO_DB . "; MAX_BUFFER_SIZE_BEFORE_REDUCING_TASKS = " . BUFFER_SIZE_BEFORE_REDUCING_TASKS);

$runtime = -time();

$dbWriteBuffer = [];
$tasksLeftInPool = 0;

$db = new Database();
$initialRowsInDb = $db->count();
$currentRowsInDb = $initialRowsInDb;

$util = new Util(); // Needs a reference from calling context. Will be destroyed if called in workers constructor to free up memory: https://github.com/krakjoe/pthreads/issues/358#issuecomment-57090365
if (BADWORD_CHECK) {

    $util->prepareBadword(BADWORD_DICT);
}

$pool = new Pool(NUMBER_OF_WORKERS_IN_POOL, CodeWorker::class, [$util]); // Insert worker and pass Util as parameter

do {

    $tasksLeftInPool = $pool->collect(function (CodeTask $codeTask) use (&$dbWriteBuffer) { // Process finished job and return number of jobs left in pool.
        if ($codeTask->isCompleted()) {

            $dbWriteBuffer = array_merge($dbWriteBuffer, (array) $codeTask->getResult());

            return true;
        } else {

            return false;
        }
    });

    $dbWriteBufferCount = count($dbWriteBuffer);

    if ($tasksLeftInPool < NUMBER_OF_CONCURRENT_TASKS && ($dbWriteBufferCount < BUFFER_SIZE_BEFORE_REDUCING_TASKS)) { // If queue becomes too large, DB write might be too slow ==> Reduce number of jobs to avoid memory issues. 
        for ($i = 0; $i < (NUMBER_OF_CONCURRENT_TASKS - $tasksLeftInPool); $i++) {

            // The work starts here
            $pool->submit(new CodeTask);
        }
    }

    if ($dbWriteBufferCount >= BUFFER_SIZE_BEFORE_SENDING_TO_DB) {

        $db->write($dbWriteBuffer);

        $dbWriteBuffer = array_slice($dbWriteBuffer, $dbWriteBufferCount);

        $currentRowsInDb = $db->count();
        
        $codesPerSecond = (int) (($currentRowsInDb - $initialRowsInDb) / ($runtime + time(true)));
        $timeRemaining = (int) ((NUMBER_OF_CODES - $currentRowsInDb) / $codesPerSecond);
        @Logger::log("Codes: " . $currentRowsInDb . "; Runtime: " . ($runtime + time(true)) . "; Average: " . $codesPerSecond . "; Remaining: " . $timeRemaining . "; Tasks running: " . $tasksLeftInPool);
    }
} while ($currentRowsInDb < NUMBER_OF_CODES);

Logger::log("Finished. Runtime: " . ($runtime + microtime(true)));
Logger::log("Shutting down...");

$pool->shutdown();

Logger::log("Done.");
